@@ -17,15 +17,9 @@ adjust_left_type <- function(x, value)
     x
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### subassign_Array_by_Lindex() and subassign_Array_by_Mindex() methods for
-### SVT_SparseArray
-###
-
 ### Adjust the type of 'value' and recycle it to the length of the
 ### subassignment M/L-index.
-.normalize_right_value <- function(value, left_type, index_len)
+normalize_right_value <- function(value, left_type, index_len)
 {
     if (length(value) == 0L)
         stop(wmsg("right value has length zero"))
@@ -33,6 +27,47 @@ adjust_left_type <- function(x, value)
     S4Vectors:::recycleVector(value, index_len)
 }
 
+adjust_right_array_dim <- function(right_array, selection_dim)
+{
+    right_dim <- unname(dim(right_array))
+    if (identical(selection_dim, right_dim))
+        return(right_array)
+    effdim_idx1 <- which(selection_dim != 1L)
+    effdim_idx2 <- which(right_dim != 1L)
+    if (!identical(selection_dim[effdim_idx1], right_dim[effdim_idx2]))
+        stop(wmsg("dimensions of right array don't ",
+                  "match dimensions of array selection"))
+    dim(right_array) <- selection_dim
+    right_array
+}
+
+### Same as 'array(data, selection_dim)' but:
+### - returns an error if 'data' is longer than the array to construct
+###   (strangely array() truncates the data in this case);
+### - issues a warning if the length of the array to construct (which is
+###   the length of the array selection in the context where array2()
+###   is used) is not a multiple of 'length(data)'.
+### Note that the error and warning messages are intentionally worded to
+### make the most sense in the context where array2() is used.
+array2 <- function(data, selection_dim)
+{
+    stopifnot(is.vector(data), is.integer(selection_dim))
+    if (length(data) > prod(selection_dim))
+        stop(wmsg("right value is longer than array selection"))
+    ans <- array(vector(typeof(data), 1L), dim=selection_dim)
+    ## Will issue "number of items to replace is not a multiple of
+    ## replacement length" warning if 'prod(selection_dim)' is not a
+    ## multiple of 'length(data)'.
+    ans[] <- data
+    ans
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### subassign_Array_by_Lindex() and subassign_Array_by_Mindex() methods for
+### SVT_SparseArray objects
+###
+
 .subassign_SVT_by_Lindex <- function(x, Lindex, value)
 {
     x <- adjust_left_type(x, value)
@@ -42,7 +77,7 @@ adjust_left_type <- function(x, value)
     if (length(Lindex) == 0L)
         return(x)
 
-    value <- .normalize_right_value(value, type(x), length(Lindex))
+    value <- normalize_right_value(value, type(x), length(Lindex))
     new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex",
                                 x@dim, x@type, x@SVT, FALSE, Lindex, value)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
@@ -61,7 +96,7 @@ setMethod("subassign_Array_by_Lindex", "SVT_SparseArray",
     if (nrow(Mindex) == 0L)
         return(x)
 
-    value <- .normalize_right_value(value, type(x), nrow(Mindex))
+    value <- normalize_right_value(value, type(x), nrow(Mindex))
     new_SVT <- SparseArray.Call("C_subassign_SVT_by_Mindex",
                                 x@dim, x@type, x@SVT, FALSE, Mindex, value)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
@@ -73,13 +108,13 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### subassign_Array_by_Nindex() method for SVT_SparseArray
+### subassign_Array_by_Nindex() method for SVT_SparseArray objects
 ###
 ### Like the 'index' argument in 'extract_array()', the 'Nindex' argument in
 ### all the functions below must be a **normalized** N-index, that is, a list
 ### with one list element per dimension in 'x' where each list element is
-### either a NULL or an integer vector of valid indices along the
-### corresponding dimension in 'x'.
+### either NULL or an integer vector of valid indices along the corresponding
+### dimension in 'x'.
 
 ### 'Rvector' is considered "short" if it can be cleanly recycled along the
 ### first (a.k.a. leftmost or innermost) dimension of the array selection.
@@ -90,7 +125,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### array selection (in other words it cannot contain zeros).
 ### Returns TRUE or FALSE indicating whether 'Rvector' is considered "short"
 ### or not.
-.is_short <- function(Rvector_len, selection_dim)
+is_short <- function(Rvector_len, selection_dim)
 {
     stopifnot(isSingleInteger(Rvector_len), is.integer(selection_dim))
     if (Rvector_len == 0L)
@@ -107,16 +142,10 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### We want to support this in the most efficient way possible so we
 ### don't actually recycle the right value at the R level. Instead we
 ### will **virtually** recycle it at the C level.
-### See .is_short() above for more information.
+### See is_short() above for more information.
 .subassign_SVT_with_short_Rvector <- function(x, Nindex, Rvector)
 {
-    stopifnot(is(x, "SVT_SparseArray"), is.list(Nindex))
-    check_svt_version(x)
-    stopifnot(is.vector(Rvector))
-
-    ## Change 'x' type if necessary.
-    new_type <- type(c(vector(type(x)), vector(type(Rvector))))
-    type(x) <- new_type
+    x <- adjust_left_type(x, Rvector)
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
@@ -124,7 +153,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
         return(x)
 
     Rvector_len <- length(Rvector)
-    stopifnot(.is_short(Rvector_len, selection_dim))
+    stopifnot(is_short(Rvector_len, selection_dim))
 
     ## Prepare 'Noffs' and 'Rvector'.
     Norder <- S4Arrays:::get_Nindex_order(Nindex)
@@ -133,27 +162,13 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     Norder1 <- Norder[[1L]]
     if (!is.null(Norder1))
         Rvector <- Rvector[((Norder1 - 1L) %% Rvector_len) + 1L]
-    storage.mode(Rvector) <- new_type
+    storage.mode(Rvector) <- type(x)
 
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_short_Rvector",
-                                x@dim, x@type, x@SVT, Noffs, Rvector)
+                                x@dim, x@type, x@SVT, FALSE, Noffs, Rvector)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.adjust_right_array_dim <- function(right_array, selection_dim)
-{
-    right_dim <- unname(dim(right_array))
-    if (identical(selection_dim, right_dim))
-        return(right_array)
-    effdim_idx1 <- which(selection_dim != 1L)
-    effdim_idx2 <- which(right_dim != 1L)
-    if (!identical(selection_dim[effdim_idx1], right_dim[effdim_idx2]))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
-    dim(right_array) <- selection_dim
-    right_array
-}
-
 .subassign_SVT_by_Noffs_with_Rarray <- function(x, Noffs, Rarray)
 {
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_Rarray",
@@ -173,7 +188,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    Rarray <- .adjust_right_array_dim(Rarray, selection_dim)
+    Rarray <- adjust_right_array_dim(Rarray, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
@@ -200,7 +215,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    y <- .adjust_right_array_dim(y, selection_dim)
+    y <- adjust_right_array_dim(y, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
@@ -217,27 +232,6 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-### Same as 'array(data, selection_dim)' but:
-### - returns an error if 'data' is longer than array to construct (strangely
-###   array() truncates the data in this case);
-### - issues a warning if the length of the array to construct (which is
-###   the length of the array selection in the context where .array2()
-###   is used) not a multiple of 'length(data)'.
-### Note that the error and warning messages are intentionally worded to
-### make the most sense in the context where .array2() is used.
-.array2 <- function(data, selection_dim)
-{
-    stopifnot(is.vector(data), is.integer(selection_dim))
-    if (length(data) > prod(selection_dim))
-        stop(wmsg("right value is longer than array selection"))
-    ans <- array(vector(typeof(data), 1L), dim=selection_dim)
-    ## Will issue "number of items to replace is not a multiple of
-    ## replacement length" warning if 'prod(selection_dim)' is not a
-    ## multiple of 'length(data)'.
-    ans[] <- data
-    ans
-}
-
 .subassign_SVT_by_Nindex <- function(x, Nindex, value)
 {
     stopifnot(is(x, "SVT_SparseArray"), is.list(Nindex))
@@ -252,12 +246,12 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
         if (any(selection_dim == 0L))
             return(x)
 
-        if (.is_short(length(value), selection_dim))
+        if (is_short(length(value), selection_dim))
             return(.subassign_SVT_with_short_Rvector(x, Nindex, value))
 
         ## Turn 'value' into an ordinary array of same dimensions as
         ## the array selection, with recycling if necessary.
-        value <- .array2(value, selection_dim)
+        value <- array2(value, selection_dim)
     }
     if (is.array(value))
         return(.subassign_SVT_with_Rarray(x, Nindex, value))
