@@ -7,9 +7,9 @@
  ****************************************************************************/
 #include "SparseArray_subassignment.h"
 
+#include "misc_utils.h"
 #include "argcheck_utils.h"
 #include "OPBufTree.h"
-#include "thread_control.h"  /* for which_max() */
 #include "leaf_utils.h"
 
 #include <limits.h>  /* for INT_MAX */
@@ -30,17 +30,6 @@ static OPBuf R_alloc_OPBuf(int buflen)
 	return opbuf;
 }
 
-static R_xlen_t *alloc_and_compute_cumprod(const int *x, int x_len)
-{
-	R_xlen_t *cumprod = (R_xlen_t *) R_alloc(x_len, sizeof(R_xlen_t));
-	R_xlen_t prod = 1;
-	for (int i = 0; i < x_len; i++) {
-		prod *= x[i];
-		cumprod[i] = prod;
-	}
-	return cumprod;
-}
-
 
 /****************************************************************************
  * subassign_leaf_by_OPBuf()
@@ -184,7 +173,7 @@ static int build_OPBufTree_from_Lindex(OPBufTree *opbuf_tree,
 	/* _free_OPBufTree(opbuf_tree) resets 'opbuf_tree->node_type'
 	   to NULL_NODE. */
 	_free_OPBufTree(opbuf_tree);
-	R_xlen_t *dimcumprod = alloc_and_compute_cumprod(x_dim, x_ndim);
+	R_xlen_t *dimcumprod = _alloc_and_compute_cumprod(x_dim, x_ndim);
 	return XLENGTH(Lindex) <= (R_xlen_t) INT_MAX ?
 		build_OPBufTree_from_Lindex1(opbuf_tree, Lindex,
 				x_dim, x_ndim, dimcumprod) :
@@ -275,12 +264,12 @@ static SEXP REC_subassign_SVT_by_OPBufTree(OPBufTree *opbuf_tree,
 }
 
 static SEXP subassign_SVT_by_OPBufTree(SEXP SVT, const int *dim, int ndim,
-		SEXPTYPE Rtype, int na_background,
+		SEXPTYPE Rtype, int bg_is_na,
 		OPBufTree *opbuf_tree, int max_opbuf_nelt, SEXP vals)
 {
 	int dim0 = dim[0];
 
-	SparseVec buf_sv = _alloc_buf_SparseVec(Rtype, dim0, na_background);
+	SparseVec buf_sv = _alloc_buf_SparseVec(Rtype, dim0, bg_is_na, 0);
 	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
 		PROTECT(buf_sv.nzvals);
 
@@ -296,7 +285,7 @@ static SEXP subassign_SVT_by_OPBufTree(SEXP SVT, const int *dim, int ndim,
 
 	/* Get 1-based rank of biggest dimension (ignoring the 1st dim).
 	   Parallel execution will be along that dimension. */
-	int pardim = which_max(dim + 1, ndim - 1) + 2;
+	int pardim = _which_max(dim + 1, ndim - 1) + 2;
 
 	SEXP ans = REC_subassign_SVT_by_OPBufTree(opbuf_tree,
 				 SVT, ndim, vals,
@@ -330,7 +319,7 @@ SEXP C_subassign_SVT_by_Lindex(
 		      "    SVT_SparseArray object and 'vals' "
 		      "must have the same type");
 
-	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+	int x_bg_is_na = _get_and_check_na_background(x_na_background,
 				"C_subassign_SVT_by_Lindex", "x_na_background");
 
 	if (!(IS_INTEGER(Lindex) || IS_NUMERIC(Lindex)))
@@ -357,7 +346,7 @@ SEXP C_subassign_SVT_by_Lindex(
 	/* --- STEP 2: Subassign SVT by OPBufTree --- */
 
 	return subassign_SVT_by_OPBufTree(x_SVT, INTEGER(x_dim), ndim,
-			Rtype, x_has_NAbg,
+			Rtype, x_bg_is_na,
 			opbuf_tree, max_opbuf_nelt, vals);
 }
 
@@ -394,7 +383,7 @@ SEXP C_subassign_SVT_by_Mindex(
 		      "    SVT_SparseArray object and 'vals' "
 		      "must have the same type");
 
-	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+	int x_bg_is_na = _get_and_check_na_background(x_na_background,
 				"C_subassign_SVT_by_Mindex", "x_na_background");
 
 	int ndim = LENGTH(x_dim);
@@ -416,16 +405,75 @@ SEXP C_subassign_SVT_by_Mindex(
 	/* --- STEP 2: Subassign SVT by OPBufTree --- */
 
 	return subassign_SVT_by_OPBufTree(x_SVT, INTEGER(x_dim), ndim,
-			Rtype, x_has_NAbg,
+			Rtype, x_bg_is_na,
 			opbuf_tree, max_opbuf_nelt, vals);
 }
 
 
 /****************************************************************************
- * new_SVT()
- * make_SVT_node()
+ * Some helpers shared by C_subassign_SVT_with_short_Rvector(),
+ * C_subassign_SVT_with_Rarray(), and C_subassign_SVT_with_SVT()
  */
 
+static int check_subassign_offs(SEXP offs, int d)
+{
+	int d2 = LENGTH(offs);
+	const int *offs_p = INTEGER(offs);
+	int prev_off = -1;
+	for (int i2 = 0; i2 < d2; i2++) {
+		int off = offs_p[i2];
+		if (off == NA_INTEGER)
+			error("subscripts contain NAs");
+		if (off < 0 || off >= d)
+			error("subscripts contain out-of-bound indices");
+		if (off <= prev_off)
+			error("SparseArray internal error in "
+			      "check_subassign_offs():\n"
+			      "    subscripts are not strictly sorted");
+		prev_off = off;
+	}
+	return d2;
+}
+
+static int check_subassign_Noffs(SEXP Noffs, const int *dim, int ndim,
+				 const int *arr_dim)
+{
+	if (!isVectorList(Noffs))  // IS_LIST() is broken
+		error("SparseArray internal error in "
+		      "check_subassign_Noffs():\n"
+		      "    'Noffs' must be a list");
+	if (LENGTH(Noffs) != ndim)
+		error("SparseArray internal error in "
+		      "check_subassign_Noffs():\n"
+		      "    'Noffs' must have one list element per "
+		      "dimension in the array to subassign");
+	for (int along = 0; along < ndim; along++) {
+		SEXP offs = VECTOR_ELT(Noffs, along);
+		int d2;  /* dim of array selection */
+		if (offs == R_NilValue) {
+			d2 = dim[along];
+		} else if (IS_INTEGER(offs)) {
+			d2 = check_subassign_offs(offs, dim[along]);
+		} else {
+			error("subscripts must be integer vectors");
+		}
+		if (d2 == 0)
+			return 1;  /* subassignment is a no-op */
+		if (arr_dim != NULL && arr_dim[along] != d2)
+			error("SparseArray internal error in "
+			      "check_subassign_Noffs():\n"
+			      "    dimensions of right array don't "
+			      "match dimensions of array selection");
+	}
+	return 0;
+}
+
+/* Offsets were already checked upfront by check_subassign_Noffs() above. */
+static int get_off(SEXP offs, int i2)
+{
+	return offs == R_NilValue ? i2 : INTEGER(offs)[i2];
+}
+
 static SEXP new_SVT(int d, SEXP SVT0)
 {
 	if (d == 0)
@@ -447,18 +495,27 @@ static SEXP new_SVT(int d, SEXP SVT0)
 	return SVT;
 }
 
-/* Used by C_subassign_SVT_with_short_Rvector() only.
-   'SVT' must be R_NilValue or a list of length 'd' ('d' cannot be 0).
-   Always returns a list of length 'd'. Can be a newly allocated list
-   or 'SVT' itself.
-   TODO: C_subassign_SVT_with_short_Rvector() should be revisited and
-   use the same allocation stretegy as C_subassign_SVT_with_Rarray()
-   and C_subassign_SVT_with_SVT(). Then make_SVT_node() can go away. */
-static inline SEXP make_SVT_node(SEXP SVT, int d, SEXP SVT0)
+static SEXP post_process_SVT(SEXP SVT, SEXP SVT0)
 {
-	if (SVT != R_NilValue && SVT != SVT0)
+	int SVT_len = LENGTH(SVT), is_empty = 1;
+	for (int i = 0; i < SVT_len; i++) {
+		if (VECTOR_ELT(SVT, i) != R_NilValue) {
+			is_empty = 0;
+			break;
+		}
+	}
+	if (is_empty)
+		return R_NilValue;
+	if (SVT0 == R_NilValue)
 		return SVT;
-	return new_SVT(d, SVT);
+	int is_noop = 1;
+	for (int i = 0; i < SVT_len; i++) {
+		if (VECTOR_ELT(SVT, i) != VECTOR_ELT(SVT0, i)) {
+			is_noop = 0;
+			break;
+		}
+	}
+	return is_noop ? SVT0 : SVT;
 }
 
 
@@ -466,316 +523,115 @@ static inline SEXP make_SVT_node(SEXP SVT, int d, SEXP SVT0)
  * C_subassign_SVT_with_short_Rvector()
  */
 
-typedef struct left_bufs_t {
-	CopyRVectorEltFUN copy_Rvector_elt_FUN;
-	SEXP Rvector;
-	int *offs;
-	SEXP precomputed_leaf;
-	int full_replacement;
-} LeftBufs;
-
-/* 'short_Rvector' must have a length >= 1.
-   'dim0' must be a multiple of 'short_Rvector' length. */
-static SEXP precompute_leaf_from_short_Rvector(
-		int dim0, SEXP index0, SEXP short_Rvector,
-		LeftBufs *left_bufs)
+static int *is_full_replacement(SEXP Noffs, const int *dim, int ndim)
 {
-	left_bufs->full_replacement = 1;
-	SEXP left_Rvector = left_bufs->Rvector;
-	int short_len = LENGTH(short_Rvector);
-	if (index0 == R_NilValue) {
-		if (short_len == dim0) {
-			left_Rvector = short_Rvector;
-		} else {
-			/* Copy a recycled version of 'short_Rvector'
-			   to 'left_bufs->Rvector'. 'left_bufs->Rvector' is
-			   of length 'dim0'. */
-			for (int i1 = 0; i1 < dim0; i1++) {
-				left_bufs->copy_Rvector_elt_FUN(short_Rvector,
-						i1 % short_len,
-						left_Rvector, i1);
-			}
-		}
-	} else {
-		for (int i1 = 0; i1 < dim0; i1++)
-			left_bufs->offs[i1] = 0;
-		/* Recycle and subassign 'short_Rvector' into 'left_Rvector'. */
-		int d2 = LENGTH(index0);
-		for (int i2 = 0; i2 < d2; i2++) {
-			int coord = INTEGER(index0)[i2];
-			if (INVALID_COORD(coord, dim0))
-				error("subscript contains "
-				      "out-of-bound indices or NAs");
-			int i1 = coord - 1;
-			left_bufs->copy_Rvector_elt_FUN(short_Rvector,
-						i2 % short_len,
-						left_Rvector, i1);
-			left_bufs->offs[i1] = 1;
-		}
-		for (int i1 = 0; i1 < dim0; i1++) {
-			if (left_bufs->offs[i1] == 0) {
-				left_bufs->full_replacement = 0;
-				break;
-			}
+	int *fully = (int *) R_alloc(ndim, sizeof(int));
+	int ok = 1;
+	for (int along = 0; along < ndim; along++) {
+		SEXP offs = VECTOR_ELT(Noffs, along);
+		ok = ok && (offs == R_NilValue || LENGTH(offs) == dim[along]);
+		fully[along] = ok;
+	}
+	return fully;
+}
+
+/* Recursive. */
+static SEXP REC_subassign_SVT_with_short_Rvector(SEXP SVT,
+		const int *dim, int ndim, SEXP Noffs, SEXP short_Rvector,
+		const int *fully, SparseVec *buf_sv)
+{
+	SEXP offs = VECTOR_ELT(Noffs, ndim - 1);
+	if (ndim == 1)
+		return _subassign_leaf_with_Rvector_block(SVT, offs,
+				offs == R_NilValue ? dim[0] : LENGTH(offs),
+				short_Rvector, 0, buf_sv);
+	SEXP ans_shared_elt = R_NilValue;
+	int use_shared = fully[ndim - 2];
+	if (use_shared) {
+		SEXP subSVT = SVT == R_NilValue ? R_NilValue :
+					VECTOR_ELT(SVT, get_off(offs, 0));
+		ans_shared_elt =
+			REC_subassign_SVT_with_short_Rvector(subSVT,
+					dim, ndim - 1, Noffs, short_Rvector,
+					fully, buf_sv);
+		if (ans_shared_elt != R_NilValue) {
+			PROTECT(ans_shared_elt);
+		} else if (fully[ndim - 1]) {
+			return R_NilValue;
 		}
 	}
-	//printf("full_replacement=%d\n", left_bufs->full_replacement);
-	return _make_leaf_from_Rvector_block(left_Rvector, 0, dim0,
-					     left_bufs->offs,
-					     left_bufs->full_replacement);
-}
-
-/* 'short_Rvector' must have a length >= 1.
-   The length of the selection along the first dimension must be a
-   multiple of 'short_Rvector' length. */
-static LeftBufs init_left_bufs(int dim0, SEXP index0, SEXP short_Rvector)
-{
-	SEXPTYPE Rtype = TYPEOF(short_Rvector);
-	LeftBufs left_bufs;
-	left_bufs.copy_Rvector_elt_FUN = _select_copy_Rvector_elt_FUN(Rtype);
-	if (left_bufs.copy_Rvector_elt_FUN == NULL)
-		error("SparseArray internal error in init_left_bufs():\n"
-		      "    short Rvector has invalid type");
-
-	R_xlen_t short_len = XLENGTH(short_Rvector);
-	int sd0 = index0 == R_NilValue ? dim0 : LENGTH(index0);
-	if (short_len == 0 || sd0 % short_len != 0)
-		error("SparseArray internal error in init_left_bufs():\n"
-		      "    invalid short Rvector length");
-
-	left_bufs.offs = (int *) R_alloc(dim0, sizeof(int));
-	left_bufs.Rvector = PROTECT(_new_Rvector0(Rtype, dim0));
-	SEXP leaf = PROTECT(
-		precompute_leaf_from_short_Rvector(
-					dim0, index0, short_Rvector,
-					&left_bufs)
-	);
-	left_bufs.precomputed_leaf = leaf;
-	UNPROTECT(2);
-	return left_bufs;
-}
-
-/* 'index0' must be either R_NilValue or an integer vector.
-   'short_Rvector' must have a length >= 1. */
-static SEXP subassign_leaf_with_short_Rvector(SEXP leaf, int dim0,
-		SEXP index0, SEXP short_Rvector,
-		LeftBufs *left_bufs)
-{
-	if (left_bufs->full_replacement || leaf == R_NilValue)
-		return left_bufs->precomputed_leaf;
-
-	SEXP left_Rvector = left_bufs->Rvector;
-	_expand_leaf(leaf, left_Rvector, 0);
-	int short_len = LENGTH(short_Rvector);
-	int d2 = LENGTH(index0);
+	int d1 = dim[ndim - 1];
+	int d2 = offs == R_NilValue ? d1 : LENGTH(offs);
+	SEXP ans = PROTECT(new_SVT(d1, SVT));
 	for (int i2 = 0; i2 < d2; i2++) {
-		int coord = INTEGER(index0)[i2];
-		if (INVALID_COORD(coord, dim0))
-			error("subscript contains "
-			      "out-of-bound indices or NAs");
-		int i1 = coord - 1;
-		/* Virtual recycling of 'short_Rvector'. */
-		left_bufs->copy_Rvector_elt_FUN(
-				short_Rvector, i2 % short_len,
-				left_Rvector, i1);
-	}
-	SEXP ans = PROTECT(
-		_make_leaf_from_Rvector_block(left_Rvector, 0, dim0,
-					      left_bufs->offs, 0)
-	);
-	if (ans != R_NilValue) {
-		/* Remove nonzeros introduced in 'left_bufs->Rvector'. */
-		SEXP ans_nzoffs = get_leaf_nzoffs(ans);
-		_fill_Rvector_subset_with_zeros(left_Rvector,
-						INTEGER(ans_nzoffs),
-						LENGTH(ans_nzoffs), 0);
+		int off = get_off(offs, i2);
+		if (use_shared) {
+			SET_VECTOR_ELT(ans, off, ans_shared_elt);
+			continue;
+		}
+		SEXP subSVT = VECTOR_ELT(ans, off);
+		SEXP ans_elt = PROTECT(
+			REC_subassign_SVT_with_short_Rvector(subSVT,
+					dim, ndim - 1, Noffs, short_Rvector,
+					fully, buf_sv)
+		);
+		SET_VECTOR_ELT(ans, off, ans_elt);
+		UNPROTECT(1);
 	}
+	if (use_shared && ans_shared_elt != R_NilValue)
+		UNPROTECT(1);
+	ans = post_process_SVT(ans, SVT);
 	UNPROTECT(1);
 	return ans;
 }
 
-/* Recursive. 'ndim' must be >= 2. */
-static SEXP REC_subassign_SVT_with_short_Rvector(SEXP SVT, SEXP SVT0,
-		const int *dim, int ndim, SEXP Nindex,
-		SEXP short_Rvector, LeftBufs *left_bufs)
-{
-	SEXP subSVT0 = R_NilValue;
-	int d1 = dim[ndim - 1];
-	SEXP Nindex_elt = VECTOR_ELT(Nindex, ndim - 1);
-	int d2 = Nindex_elt == R_NilValue ? d1 : LENGTH(Nindex_elt);
-	//printf("ndim = %d: d2 = %d\n", ndim, d2);
-	for (int i2 = 0; i2 < d2; i2++) {
-		int i1;
-		if (Nindex_elt == R_NilValue) {
-			i1 = i2;
-		} else {
-			int coord = INTEGER(Nindex_elt)[i2];
-			if (INVALID_COORD(coord, d1))
-				error("subscript contains "
-				      "out-of-bound indices or NAs");
-			i1 = coord - 1;
-		}
-		//printf("ndim = %d: i1 = %d i2 = %d\n", ndim, i1, i2);
-		SEXP subSVT = VECTOR_ELT(SVT, i1);
-		if (ndim == 2) {
-			subSVT = PROTECT(
-				subassign_leaf_with_short_Rvector(
-					subSVT, dim[0],
-					VECTOR_ELT(Nindex, 0), short_Rvector,
-					left_bufs)
-			);
-		} else {
-			if (SVT0 != R_NilValue)
-				subSVT0 = VECTOR_ELT(SVT0, i1);
-			subSVT = PROTECT(
-				make_SVT_node(subSVT, dim[ndim - 2], subSVT0)
-			);
-			subSVT = PROTECT(
-				REC_subassign_SVT_with_short_Rvector(
-					subSVT, subSVT0,
-					dim, ndim - 1, Nindex,
-					short_Rvector, left_bufs)
-			);
-		}
-		SET_VECTOR_ELT(SVT, i1, subSVT);
-		UNPROTECT(ndim == 2 ? 1 : 2);
-	}
-	int is_empty = 1;
-	for (int i1 = 0; i1 < d1; i1++) {
-		if (VECTOR_ELT(SVT, i1) != R_NilValue) {
-			is_empty = 0;
-			break;
-		}
-	}
-	return is_empty ? R_NilValue : SVT;
-}
-
 /* --- .Call ENTRY POINT ---
-   'Nindex' must be an N-index, that is, a list of integer vectors (or NULLs),
-   one along each dimension in the array. */
-SEXP C_subassign_SVT_with_short_Rvector(SEXP x_dim, SEXP x_type, SEXP x_SVT,
-		SEXP Nindex, SEXP Rvector)
+   'Noffs' must be a list of integer vectors (or NULLs), one along each
+   dimension in the arrays. Each non-NULL list element must contain valid
+   offsets (i.e. zero-based indices) along the corresponding dimension in 'x'.
+   The offsets must be sorted in **strictly** ascending order. */
+SEXP C_subassign_SVT_with_short_Rvector(
+		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
+		SEXP Noffs, SEXP Rvector)
 {
-	SEXPTYPE Rtype = _get_and_check_Rtype_from_Rstring(x_type,
-				"C_subassign_SVT_with_short_Rvector", "x_type");
-	if (TYPEOF(Rvector) != Rtype)
+	SEXPTYPE x_Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+			     "C_subassign_SVT_with_short_Rvector", "x_type");
+	if (TYPEOF(Rvector) != x_Rtype)
 		error("SparseArray internal error in "
 		      "C_subassign_SVT_with_short_Rvector():\n"
 		      "    SVT_SparseArray object and 'Rvector' "
 		      "must have the same type");
 
+	int x_bg_is_na = _get_and_check_na_background(x_na_background,
+			     "C_subassign_SVT_with_short_Rvector",
+			     "x_na_background");
+
 	const int *dim = INTEGER(x_dim);
 	int ndim = LENGTH(x_dim);
-	for (int along = 0; along < ndim; along++)
-		if (dim[along] == 0)
-			return x_SVT;  /* no-op */
+	if (check_subassign_Noffs(Noffs, dim, ndim, NULL))
+		return x_SVT;  /* no-op */
 
-	int dim0 = dim[0];
-	SEXP index0 = VECTOR_ELT(Nindex, 0);
+	const int *fully = is_full_replacement(Noffs, dim, ndim);
 
-	LeftBufs left_bufs = init_left_bufs(dim0, index0, Rvector);
-	PROTECT(left_bufs.Rvector);
-	PROTECT(left_bufs.precomputed_leaf);
-
-	if (ndim == 1) {
-		SEXP ans = subassign_leaf_with_short_Rvector(
-					x_SVT, dim0,
-					index0, Rvector, &left_bufs);
-		UNPROTECT(2);
-		return ans;
-	}
-
-	SEXP ans = PROTECT(make_SVT_node(x_SVT, dim[ndim - 1], x_SVT));
-	ans = REC_subassign_SVT_with_short_Rvector(ans, x_SVT,
-					dim, ndim, Nindex,
-					Rvector, &left_bufs);
-	UNPROTECT(3);
+	/* _subassign_leaf_with_Rvector_block(), the workhorse behind
+	   REC_subassign_SVT_with_short_Rvector(), does not always need
+	   'buf_sv.nzvals' and is able to allocate it the first time it
+	   needs it (if it ever needs it). So we set the 'nzoffs_only'
+	   argument to 1 in our _alloc_buf_SparseVec() call below. */
+	SparseVec buf_sv = _alloc_buf_SparseVec(x_Rtype, dim[0], x_bg_is_na, 1);
+	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
+		buf_sv.nzvals = PROTECT(
+			allocVector(buf_sv.Rtype, (R_xlen_t) buf_sv.len)
+		);
+	SEXP ans = REC_subassign_SVT_with_short_Rvector(x_SVT, dim, ndim,
+						Noffs, Rvector,
+						fully, &buf_sv);
+	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
+		UNPROTECT(1);
 	return ans;
 }
 
 
-/****************************************************************************
- * Some helpers shared between C_subassign_SVT_with_Rarray() and
- * C_subassign_SVT_with_SVT()
- */
-
-static int check_offs(SEXP offs, int d)
-{
-	int n = LENGTH(offs);
-	const int *offs_p = INTEGER(offs);
-	int prev_off = -1;
-	for (int i = 0; i < n; i++) {
-		int off = offs_p[i];
-		if (off == NA_INTEGER)
-			error("subscripts contain NAs");
-		if (off < 0 || off >= d)
-			error("subscripts contain out-of-bound indices");
-		if (off <= prev_off)
-			error("SparseArray internal error in check_offs():\n"
-			      "    subscripts are not strictly sorted");
-		prev_off = off;
-	}
-	return n;
-}
-
-static int check_Noffs(SEXP Noffs, const int *dim, const int *arr_dim, int ndim)
-{
-	if (LENGTH(Noffs) != ndim)
-		error("SparseArray internal error in check_Noffs():\n"
-		      "    'Noffs' must have one list element per "
-		      "dimension in the array to subassign");
-	for (int along = 0; along < ndim; along++) {
-		if (dim[along] == 0)
-			return 1;  /* subassignment is a no-op */
-		SEXP offs = VECTOR_ELT(Noffs, along);
-		int doas;  /* dim of array selection */
-		if (offs == R_NilValue) {
-			doas = dim[along];
-		} else if (IS_INTEGER(offs)) {
-			doas = check_offs(offs, dim[along]);
-		} else {
-			error("subscripts must be integer vectors");
-		}
-		if (doas == 0)
-			return 1;  /* subassignment is a no-op */
-		if (arr_dim[along] != doas)
-			error("SparseArray internal error in check_Noffs():\n"
-			      "    dimensions of right array don't "
-			      "match dimensions of array selection");
-	}
-	return 0;
-}
-
-/* Offsets were already checked upfront by check_Noffs() above. */
-static int get_off(SEXP offs, int i)
-{
-	return offs == R_NilValue ? i : INTEGER(offs)[i];
-}
-
-static SEXP post_process_ans(SEXP ans, SEXP SVT)
-{
-	int ans_len = LENGTH(ans), is_empty = 1;
-	for (int i = 0; i < ans_len; i++) {
-		if (VECTOR_ELT(ans, i) != R_NilValue) {
-			is_empty = 0;
-			break;
-		}
-	}
-	if (is_empty)
-		return R_NilValue;
-	if (SVT == R_NilValue)
-		return ans;
-	int is_noop = 1;
-	for (int i = 0; i < ans_len; i++) {
-		if (VECTOR_ELT(ans, i) != VECTOR_ELT(SVT, i)) {
-			is_noop = 0;
-			break;
-		}
-	}
-	return is_noop ? SVT : ans;
-}
-
-
 /****************************************************************************
  * C_subassign_SVT_with_Rarray()
  */
@@ -797,10 +653,11 @@ static SEXP check_Rarray(SEXP Rarray, int ndim, SEXPTYPE x_Rtype)
 	return Rarray_dim;
 }
 
+/* Recursive. */
 static SEXP REC_subassign_SVT_with_Rsubarr(SEXP SVT,
 		const int *dim, int ndim, SEXP Noffs,
 		SEXP Rarray, R_xlen_t arr_offset, const R_xlen_t *subarr_lens,
-		SparseVec *buf_sv)
+		SparseVec *buf_sv, int pardim)
 {
 	SEXP offs = VECTOR_ELT(Noffs, ndim - 1);
 	if (ndim == 1)
@@ -811,19 +668,25 @@ static SEXP REC_subassign_SVT_with_Rsubarr(SEXP SVT,
 	int d2 = offs == R_NilValue ? d1 : LENGTH(offs);
 	SEXP ans = PROTECT(new_SVT(d1, SVT));
 	R_xlen_t offset_inc = subarr_lens[ndim - 2];
-	for (int i2 = 0; i2 < d2; i2++, arr_offset += offset_inc) {
-		int i1 = get_off(offs, i2);
-		SEXP subSVT = VECTOR_ELT(ans, i1);
+	/* Parallel execution along the biggest dimension only. */
+	//Nope, not doing this yet! Major issue is that the 'buf_sv->nzvals'
+	//and 'buf_sv->nzoffs' arrays are shared by all threads which will
+	//cause a major disaster!!!
+	//#pragma omp parallel for schedule(static) if(ndim == pardim)
+	for (int i2 = 0; i2 < d2; i2++) {
+		int off = get_off(offs, i2);
+		SEXP subSVT = VECTOR_ELT(ans, off);
+		R_xlen_t subarr_offset = arr_offset + offset_inc * i2;
 		SEXP ans_elt = PROTECT(
 			REC_subassign_SVT_with_Rsubarr(subSVT,
 					dim, ndim - 1, Noffs,
-					Rarray, arr_offset, subarr_lens,
-					buf_sv)
+					Rarray, subarr_offset, subarr_lens,
+					buf_sv, pardim)
 		);
-		SET_VECTOR_ELT(ans, i1, ans_elt);
+		SET_VECTOR_ELT(ans, off, ans_elt);
 		UNPROTECT(1);
 	}
-	ans = post_process_ans(ans, SVT);
+	ans = post_process_SVT(ans, SVT);
 	UNPROTECT(1);
 	return ans;
 }
@@ -834,15 +697,14 @@ static SEXP REC_subassign_SVT_with_Rsubarr(SEXP SVT,
    'Noffs' must be a list of integer vectors (or NULLs), one along each
    dimension in the arrays. Each non-NULL list element must contain valid
    offsets (i.e. zero-based indices) along the corresponding dimension in 'x'.
-   IMPORTANT: The offsets must be sorted in **strictly** ascending order.
-   This is not checked! */
+   The offsets must be sorted in **strictly** ascending order. */
 SEXP C_subassign_SVT_with_Rarray(
 		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
 		SEXP Noffs, SEXP Rarray)
 {
 	SEXPTYPE x_Rtype = _get_and_check_Rtype_from_Rstring(x_type,
 			     "C_subassign_SVT_with_Rarray", "x_type");
-	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+	int x_bg_is_na = _get_and_check_na_background(x_na_background,
 			     "C_subassign_SVT_with_Rarray", "x_na_background");
 
 	int ndim = LENGTH(x_dim);
@@ -850,16 +712,28 @@ SEXP C_subassign_SVT_with_Rarray(
 
 	const int *dim = INTEGER(x_dim);
 	const int *arr_dim = INTEGER(Rarray_dim);
-	if (check_Noffs(Noffs, dim, arr_dim, ndim))
+	if (check_subassign_Noffs(Noffs, dim, ndim, arr_dim))
 		return x_SVT;  /* no-op */
 
-	SparseVec buf_sv = _alloc_buf_SparseVec(x_Rtype, dim[0], x_has_NAbg);
+	/* _subassign_leaf_with_Rvector_block(), the workhorse behind
+	   REC_subassign_SVT_with_Rsubarr(), does not always need
+	   'buf_sv.nzvals' and is able to allocate it the first time it
+	   needs it (if it ever needs it). So we set the 'nzoffs_only'
+	   argument to 1 in our _alloc_buf_SparseVec() call below. */
+	SparseVec buf_sv = _alloc_buf_SparseVec(x_Rtype, dim[0], x_bg_is_na, 1);
 	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
-		PROTECT(buf_sv.nzvals);
-	R_xlen_t *subarr_lens = alloc_and_compute_cumprod(arr_dim, ndim);
+		buf_sv.nzvals = PROTECT(
+			allocVector(buf_sv.Rtype, (R_xlen_t) buf_sv.len)
+		);
+	R_xlen_t *subarr_lens = _alloc_and_compute_cumprod(arr_dim, ndim);
+
+	/* Get 1-based rank of biggest dimension (ignoring the 1st dim).
+	   Parallel execution will be along that dimension. */
+	int pardim = _which_max(arr_dim + 1, ndim - 1) + 2;
+
 	SEXP ans = REC_subassign_SVT_with_Rsubarr(x_SVT, dim, ndim, Noffs,
 						  Rarray, 0, subarr_lens,
-						  &buf_sv);
+						  &buf_sv, pardim);
 	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
 		UNPROTECT(1);
 	return ans;
@@ -870,6 +744,7 @@ SEXP C_subassign_SVT_with_Rarray(
  * C_subassign_SVT_with_SVT()
  */
 
+/* Recursive. */
 static SEXP REC_subassign_SVT1_with_SVT2(
 		SEXP SVT1, const int *dim1, int ndim, SEXP Noffs,
 		SEXP SVT2, const int *dim2, SparseVec *buf_sv)
@@ -884,8 +759,8 @@ static SEXP REC_subassign_SVT1_with_SVT2(
 	int d2 = dim2[ndim - 1];
 	SEXP ans = PROTECT(new_SVT(d1, SVT1));
 	for (int i2 = 0; i2 < d2; i2++) {
-		int i1 = get_off(offs, i2);
-		SEXP subSVT1 = VECTOR_ELT(ans, i1);
+		int off = get_off(offs, i2);
+		SEXP subSVT1 = VECTOR_ELT(ans, off);
 		SEXP subSVT2 = SVT2 == R_NilValue ? R_NilValue :
 						    VECTOR_ELT(SVT2, i2);
 		SEXP ans_elt = PROTECT(
@@ -893,10 +768,10 @@ static SEXP REC_subassign_SVT1_with_SVT2(
 					dim1, ndim - 1, Noffs,
 					subSVT2, dim2, buf_sv)
 		);
-		SET_VECTOR_ELT(ans, i1, ans_elt);
+		SET_VECTOR_ELT(ans, off, ans_elt);
 		UNPROTECT(1);
 	}
-	ans = post_process_ans(ans, SVT1);
+	ans = post_process_SVT(ans, SVT1);
 	UNPROTECT(1);
 	return ans;
 }
@@ -907,8 +782,7 @@ static SEXP REC_subassign_SVT1_with_SVT2(
    'Noffs' must be a list of integer vectors (or NULLs), one along each
    dimension in the arrays. Each non-NULL list element must contain valid
    offsets (i.e. zero-based indices) along the corresponding dimension in 'x'.
-   IMPORTANT: The offsets must be sorted in **strictly** ascending order.
-   This is not checked! */
+   The offsets must be sorted in **strictly** ascending order. */
 SEXP C_subassign_SVT_with_SVT(
 		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
 		SEXP Noffs,
@@ -916,20 +790,20 @@ SEXP C_subassign_SVT_with_SVT(
 {
 	SEXPTYPE x_Rtype = _get_and_check_Rtype_from_Rstring(x_type,
 			     "C_subassign_SVT_with_SVT", "x_type");
-	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+	int x_bg_is_na = _get_and_check_na_background(x_na_background,
 			     "C_subassign_SVT_with_SVT", "x_na_background");
 	SEXPTYPE y_Rtype = _get_and_check_Rtype_from_Rstring(y_type,
 			     "C_subassign_SVT_with_SVT", "y_type");
-	int y_has_NAbg = _get_and_check_na_background(y_na_background,
+	int y_bg_is_na = _get_and_check_na_background(y_na_background,
 			     "C_subassign_SVT_with_SVT", "y_na_background");
 	if (x_Rtype != y_Rtype)
 		error("SparseArray internal error in "
 		      "C_subassign_SVT_with_SVT():\n"
 		      "    x_Rtype != y_Rtype");
-	if (x_has_NAbg != y_has_NAbg)
+	if (x_bg_is_na != y_bg_is_na)
 		error("SparseArray internal error in "
 		      "C_subassign_SVT_with_SVT():\n"
-		      "    x_has_NAbg != y_has_NAbg");
+		      "    x_bg_is_na != y_bg_is_na");
 
 	int ndim = LENGTH(x_dim);
 	if (LENGTH(y_dim) != ndim)
@@ -937,11 +811,11 @@ SEXP C_subassign_SVT_with_SVT(
 		      "C_subassign_SVT_with_SVT():\n"
 		      "    LENGTH(x_dim) != LENGTH(y_dim)");
 
-	if (check_Noffs(Noffs, INTEGER(x_dim), INTEGER(y_dim), ndim))
+	if (check_subassign_Noffs(Noffs, INTEGER(x_dim), ndim, INTEGER(y_dim)))
 		return x_SVT;  /* no-op */
 
 	SparseVec buf_sv = _alloc_buf_SparseVec(x_Rtype, INTEGER(x_dim)[0],
-						x_has_NAbg);
+						x_bg_is_na, 0);
 	if (IS_STRSXP_OR_VECSXP(buf_sv.Rtype))
 		PROTECT(buf_sv.nzvals);
 	SEXP ans = REC_subassign_SVT1_with_SVT2(x_SVT,
@@ -984,7 +858,7 @@ static long long init_NindexIterator(NindexIterator *Nindex_iter,
 		const int *dim, int ndim, SEXP Nindex, int margin)
 {
 	long long selection_len;
-	int along, doas;
+	int along, d2;
 	SEXP Nindex_elt;
 
 	if (!isVectorList(Nindex) || LENGTH(Nindex) != ndim)
@@ -1001,16 +875,16 @@ static long long init_NindexIterator(NindexIterator *Nindex_iter,
 	for (along = 0; along < ndim; along++) {
 		Nindex_elt = VECTOR_ELT(Nindex, along);
 		if (Nindex_elt == R_NilValue) {
-			doas = dim[along];
+			d2 = dim[along];
 		} else if (IS_INTEGER(Nindex_elt)) {
-			doas = LENGTH(Nindex_elt);
+			d2 = LENGTH(Nindex_elt);
 		} else {
 			error("subscripts must be integer vectors");
 		}
-		selection_len *= doas;
+		selection_len *= d2;
 		if (along < margin)
 			continue;
-		Nindex_iter->selection_dim[along - margin] = doas;
+		Nindex_iter->selection_dim[along - margin] = d2;
 		Nindex_iter->selection_midx_buf[along - margin] = 0;
 	}
 	Nindex_iter->selection_len = selection_len;
