@@ -7,6 +7,7 @@
 
 #include "Rvector_utils.h"
 #include "coerceVector2.h"
+#include "SparseVec_subsetting.h"
 #include "SparseVec_subassignment.h"
 
 #include <string.h>  /* for memcpy() */
@@ -405,10 +406,8 @@ SEXP _coerce_naleaf(SEXP leaf, SEXPTYPE new_Rtype, int *warn,
 
 
 /****************************************************************************
- * _subassign_leaf_with_Rvector_block()
- * _subassign_leaf_with_Rvector_subset()
- * _subassign_leaf_with_Rvector_xsubset()
- * _subassign_leaf_with_leaf()
+ * _subset_leaf()
+ * _subset_leaf_into_Rvector_block()
  */
 
 static const int *get_offs0(SEXP offs, int n, int dim0)
@@ -425,26 +424,128 @@ static const int *get_offs0(SEXP offs, int n, int dim0)
 	return INTEGER(offs);
 }
 
+/* Can be used on a NULL or lacunar leaf.
+   'offs' must be NULL or an array of 'buf_sv->len' offsets that are >= 0
+   and < 'dim0'. */
+SEXP _subset_leaf(SEXP leaf, int dim0, SEXP offs,
+		  SparseVec *buf_sv, int *lookup_table)
+{
+	if (leaf == R_NilValue || offs == R_NilValue)
+		return leaf;
+	/* Note that the background value does not matter in the context
+	   of N-index subsetting, because _subset_SV() -- the workhorse
+	   behind this form of subsetting -- does not make any use of it. */
+	const SparseVec sv = leaf2SV(leaf, buf_sv->Rtype,
+				     dim0, 0);  // we set 'bg_is_na' to 0
+	_subset_SV(&sv, INTEGER(offs), buf_sv, lookup_table);
+	return SV2leaf(buf_sv);
+}
+
+/* Can be used on a NULL or lacunar leaf.
+   'offs' must be NULL or an array of 'n' offsets that are >= 0 and < 'dim0'.
+   The destination is the block of 'n' elements in 'Rvector' that starts
+   at offset 'block_offset'.
+   IMPORTANT: The destination block is assumed to have been previously
+   initialized with zeros (or NAs). */
+void _subset_leaf_into_Rvector_block(SEXP leaf, SEXPTYPE Rtype, int dim0,
+		SEXP offs, int n,
+		SEXP Rvector, R_xlen_t block_offset, int *lookup_table)
+{
+	const int *offs0 = get_offs0(offs, n, dim0);
+	if (leaf == R_NilValue)
+		return;
+	/* Note that the background value does not matter in the context
+	   of N-index subsetting, because _subset_SV_into_Rvector_block()
+	   -- the workhorse behind this form of subsetting -- does not make
+	   any use of it. */
+	const SparseVec sv = leaf2SV(leaf, Rtype,
+				     dim0, 0);  // we set 'bg_is_na' to 0
+	_subset_SV_into_Rvector_block(&sv, offs0, n,
+			Rvector, block_offset, lookup_table);
+	return;
+}
+
+
+/****************************************************************************
+ * _subassign_leaf_with_Rvector_block()
+ * _subassign_leaf_with_Rvector_subset()
+ * _subassign_leaf_with_Rvector_xsubset()
+ * _subassign_leaf_with_leaf()
+ */
+
+/* TODO: Maybe move this to SparseVec.c next to _alloc_buf_SparseVec(). */
+static int alloc_SV_nzvals_if_needed(SparseVec *sv)
+{
+	if (sv->nzvals != NULL)
+		return 0;
+	size_t Rtype_size = _get_Rtype_size(sv->Rtype);
+	if (Rtype_size == 0)
+		error("SparseArray internal error in "
+		      "alloc_SV_nzvals_if_needed():\n"
+		      "    type \"%s\" is not supported", type2char(sv->Rtype));
+	sv->nzvals = R_alloc(sv->len, Rtype_size);
+	return 1;
+}
+
 /* Can be used on a NULL or lacunar leaf.
    'offs' must be NULL or an array of 'n' offsets (non-negative integers)
    that are sorted in strictly ascending order. The last offset in the
-   array must be < 'buf_sv->len'. */
+   array must be < 'buf_sv->len'.
+   About the length of 'Rvector':
+   - If 'block_offset != 0' then its length must be >= 'block_offset + n'.
+   - If 'block_offset == 0' then 'Rvector' can be of any length (except 0
+     unless 'n' is also 0) and will be recycled if its length is < 'n'. */
 SEXP _subassign_leaf_with_Rvector_block(SEXP leaf, SEXP offs, int n,
 		SEXP Rvector, R_xlen_t block_offset, SparseVec *buf_sv)
 {
 	const int *offs0 = get_offs0(offs, n, buf_sv->len);
 	if (leaf == R_NilValue) {
+		/* Taking the shortcut is not really needed because using
+		   _write_Rvector_block_to_SV() can handle this case. It's
+		   just that _make_leaf_from_Rvector_block() or
+		   _make_naleaf_from_Rvector_block() don't
+		   need 'buf_sv->nzvals'. This gives us the opportunity to
+		   use a 'buf_sv' where 'buf_sv->nzvals' is not allocated
+		   yet (i.e. is set to NULL) and to delay allocation until
+		   it's actually needed.
+		   Note that there's not measurable speed difference
+		   between taking the shortcut or not, despite the shortcut
+		   copying the nzvals data only once (directly from 'Rvector'
+		   to the returned leaf) instead of twice (first by
+		   _write_Rvector_block_to_SV() from 'Rvector'
+		   to 'buf_sv->nzvals', then by SV2leaf()
+		   from 'buf_sv->nzvals' to the returned leaf). */
+		int use_shortcut = offs0 == NULL &&
+				   (block_offset != 0 || LENGTH(Rvector) >= n);
+		if (use_shortcut) {
+			SEXP ans;
+			if (buf_sv->bg_is_na) {
+				ans = _make_naleaf_from_Rvector_block(
+						Rvector, block_offset, n,
+						buf_sv->nzoffs, 1);
+			} else {
+				ans = _make_leaf_from_Rvector_block(
+						Rvector, block_offset, n,
+						buf_sv->nzoffs, 1);
+			}
+			return ans;
+		}
+		alloc_SV_nzvals_if_needed(buf_sv);
 		_write_Rvector_block_to_SV(Rvector, block_offset,
 					   offs0, n, buf_sv);
 	} else {
+		alloc_SV_nzvals_if_needed(buf_sv);
 		const SparseVec sv = leaf2SV(leaf, buf_sv->Rtype,
 					     buf_sv->len,
-					     buf_sv->na_background);
+					     buf_sv->bg_is_na);
 		int neffrep;
-		if (offs0 == NULL) {
+		if (n == buf_sv->len) {
+			/* Full replacement.
+			   We don't need 'offs0' even if it's not NULL. */
 			neffrep = _subassign_full_SV_with_Rvector_block(&sv,
 					     Rvector, block_offset, buf_sv);
 		} else {
+			/* 'offs0' guaranteed to be != NULL. */
 			neffrep = _subassign_SV_with_Rvector_block(&sv,
 					     offs0, n,
 					     Rvector, block_offset, buf_sv);
@@ -472,7 +573,7 @@ SEXP _subassign_leaf_with_Rvector_subset(SEXP leaf, const int *offs, int n,
 	} else {
 		const SparseVec sv = leaf2SV(leaf, buf_sv->Rtype,
 					     buf_sv->len,
-					     buf_sv->na_background);
+					     buf_sv->bg_is_na);
 		int neffrep = _subassign_SV_with_Rvector_subset(&sv,
 					     offs, n,
 					     Rvector, selection, buf_sv);
@@ -500,14 +601,16 @@ SEXP _subassign_leaf_with_leaf(SEXP leaf1, SEXP offs, int n,
 		SEXP leaf2, SparseVec *buf_sv)
 {
 	const int *offs0 = get_offs0(offs, n, buf_sv->len);
-	if ((leaf1 == R_NilValue && leaf2 == R_NilValue) || offs0 == NULL)
+	/* 'n == buf_sv->len' indicates full replacement. */
+	if ((leaf1 == R_NilValue && leaf2 == R_NilValue) || n == buf_sv->len)
 		return leaf2;
+	/* 'offs0' guaranteed to be != NULL. */
 	const SparseVec sv1 = leaf2SV(leaf1, buf_sv->Rtype,
 				      buf_sv->len,
-				      buf_sv->na_background);
+				      buf_sv->bg_is_na);
 	const SparseVec sv2 = leaf2SV(leaf2, buf_sv->Rtype,
 				      n,
-				      buf_sv->na_background);
+				      buf_sv->bg_is_na);
 	int neffrep = _subassign_SV_with_SV(&sv1, offs0, &sv2, buf_sv);
 	//printf("n = %d / neffrep = %d\n", n, neffrep);
 	if (neffrep == 0)
